#!/usr/bin/env python
"""
    Copyright 2013 Marcel Kerkveld

    This file is part of Vandaag.

    Vandaag is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


from vandaag_database import *
from vandaag_table import tabulate
import datetime

########################################################
# reports
########################################################
def today(con, args):
    "No arguments needed, prints all tasks set for today"
    print('')
    heads = ['Id','Project','Description','Due']
    rows = [[tas[h] for h in heads  ] for tas in tasksreport(con, day=datetime.date.today())]
    print(tabulate(rows,headers=heads, missingval="?"))


def projects(con,arg):
    "No argument neeeded, prints all projects"
    print('')
    print(tabulate(projectreport(con),headers=['Project'], missingval="?"))


def tasks(con, arg):
    """Without argument, this will print all tasks, categorised by project
Optional argument id projectid or project name, to only print tasks assigned to that project"""
    print('')
    try:
        project = arg[0]
    except:
        project=''
    heads = ['Project','Id','Description','Due']
    rows = [[tas[h] for h in heads  ] for tas in tasksreport(con, project=project)]
    res = {}
    for r in rows:
        res.setdefault(r[0], []).append(r[1:])
    for key,value in res.iteritems():
        print('\033[1m' + key + '\033[0m')
        print(tabulate(value,headers=heads[1:], missingval="?"))
        print('')


def setvandaag(con,arg):
    "Set the indicated tasks for today. Arguments are Task id numbers"
    for i in arg:
        changeFieldTask(con,i,'day',datetime.date.today())

def setmorgen(con,arg):
    """Set the indicated tasks for tomorrow.
    Usage: <opt: TaskID>"""
    for i in arg:
        changeFieldTask(con,i,'day',datetime.date.today()+datetime.timedelta(days=1))


def setDue(con,arg):
    """Set date due.
    Usage: <TaskID> YYYY-MM-DD"""

    try:
        changeFieldTask(con,int(arg[0]),'due',arg[1])
    except:
        print('Due date not set')

def setDone(con,arg):
    """Set date due.
    Usage: <TaskID>"""

    try:
        changeFieldTask(con,int(arg[0]),'done',1)
    except:
        print('Due date not set')


# create things
def createProject(con,arg):
    "create a new project"
    try:
        p = insertProject(con,arg[0])
    except:
        print('Project not added')
    else:
        print('Project added')
    return p


def createTask(con,arg):
    "create a new task"
    try:
        pr = arg.pop(0)
        project = projectid(con,project=pr)[0][0]
    except LookupError:
        default = 'N'
        us = raw_input("Project doesn\'t exist, create? [y/N]") or default
        if us.upper()=='Y':
            project = createProject(con,[pr])
        else: exit()

    # to do if project not exists, create?
    name = ' '.join(arg)
    a = insertTask(con,name, project)
    print('Task {0} created'.format(a))

def delProject(con,arg):
    'delete a project'
    default = 'N'
    us = raw_input("Are you sure? [y/N]") or default
    if us.upper()=='Y':
        removeProject(con,arg[0])

def delTask(con,arg):
    'delete a specific task'
    default = 'N'
    us = raw_input("Are you sure? [y/N]") or default
    if us.upper()=='Y':
        removeTask(con,arg[0])

def cleanTasksDone(con,arg=''):
    'Deletes all tasks where done = 1'
    default = 'N'
    us = raw_input("Are you sure? [y/N]") or default
    if us.upper() == 'Y':
        cleanTask(con)

commands = {
            'now'           : today,
            'projects'      : projects,
            'tasks'         : tasks,
            '+'             : setvandaag,
            '+today'        : setvandaag,
            '+tomorrow'     : setmorgen,
            '+task'         : createTask,
            '-task'         : delTask,
            '+proj'         : createProject,
            '-proj'         : delProject,
            'due'           : setDue,
            'done'          : setDone,
            'clean'         : cleanTasksDone,
            }
