#!/usr/bin/env python
"""
    Copyright 2013 Marcel Kerkveld

    This file is part of Vandaag.

    Vandaag is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import sqlite3 as lite
from os.path import exists
# import datetime

def insertSQL(con,sql,values):
    # for insert to be reusedfunction
    with con:
        #cur = con.cursor()
        #cur.execute(sql,values)
        try:
            #con.row_factory = lite.Row
            cur = con.cursor()
            cur.execute(sql,values)
            return cur.lastrowid
        except lite.Error, e:
            if con:
                con.rollback()
            raise lite.Error, e


def runquery(con,sql,values):
    with con:
        cur = con.cursor()
        cur.execute(sql,values)

        rows = cur.fetchall()
        return rows

def connectdb(db):
    """
    Creates a connection. If db doesn't exist, it'll be created.
    """
    ex = not exists(db) # set ex false if not already exists, used later to create structure
    con = lite.connect(db)
    if ex:
        # create tables
        print('Creating database...')
        with con:
            cur = con.cursor()

            cur.execute("CREATE TABLE Projects(name TEXT UNIQUE)")  #project table
            cur.execute(                                                            # Tasks table
                        '''CREATE TABLE Tasks(
                                                name TEXT,
                                                day DATETIME default current_timestamp,
                                                due DATETIME,
                                                done INTEGER default 0,
                                                project INTEGER,
                                                interval TEXT,
                                                workingday INTEGER default 0,
                                                FOREIGN KEY(project) REFERENCES Projects(rowid)
                                                 )'''
                                                    )

    return con

#insert functions
def insertProject(con,name):
    # creates project and name and returns rowid
    return insertSQL(
                    con,
                    "INSERT INTO Projects VALUES(?);",(name,)
                    )

def insertTask(con,name, project, day=None, due=None, done=0, interval=None, workingday=0):

        return insertSQL(
                        con,
                        "INSERT INTO Tasks VALUES(?,?,?,?,?,?,?);",(name, day, due, done, project, interval, workingday)
                        )
###########################################################
# Changefield
###########################################################
def changeFieldTask(con,ID,field,value):
    with con:
        cur = con.cursor()
        cur.execute("UPDATE Tasks SET {field}=? WHERE rowid=?".format(field=field), (value, ID))
        con.commit()
###########################################################
# Remove
###########################################################
def removeProject(con,project):
    with con:
        cur = con.cursor()
        cur.execute("DELETE FROM Projects WHERE name=?", (project,))
        con.commit()
        cur.execute("DELETE FROM Tasks WHERE project not in (SELECT rowid from Projects)")
        con.commit()

def removeTask(con,rid):
    with con:
        cur = con.cursor()
        cur.execute("DELETE FROM Tasks WHERE rowid=?", (rid,))
        con.commit()

def cleanTask(con):
    with con:
        cur = con.cursor()
        cur.execute("DELETE FROM Tasks WHERE done=1",())
        con.commit()

###########################################################
# queries
###########################################################
def projectreport(con,project=""):
    """
    Gives list of tuples, containing all existing projects.
    """
    params=[]
    p = {}
    if project:
        try:
            pr = int(project)
            params.append("rowid=:project")
            p['Project'] = pr
        except:
            params.append("Projects.name=:project")
            p['project'] = project
    req = ' AND '.join(params)
    if req:
        req = 'WHERE {0}'.format(req)
    return runquery(con,"""SELECT name from Projects {req};""".format(req=req),p)

def projectid(con,project):
    return runquery(con,"""SELECT rowid from Projects WHERE name='{0}'""".format(project),{})

def tasksreport(con, project = None,projectid=None,done=None,day=None):
    '''returns dictionary with heads as keys and found value as value'''
    params=[]
    p = {}
    if project:
        params.append("Projects.name=:project")
        p['project'] = project
    if projectid:
        pr = int(projectid)
        params.append("Projects.rowid=:pid")
        p['pid'] = pr
    try:
        p['task']=int(done)
        params.append("Tasks.done=:task")             # if done is not an integer or a convertable string, then error
    except: pass
    if day:
        p['day']=day
        params.append('Tasks.day=:day')

    req = ' AND '.join(params)
    if req:
        req = ' AND {0}'.format(req)

    heads = ('Id', 'Project','Description','Day','Due')

    rows = runquery(con,"""SELECT TASKS.rowid, Projects.name, Tasks.name, day, date(Tasks.due)
                            FROM Projects, Tasks
                            WHERE Projects.rowid = Tasks.project AND done=0{req}""".format(req=req),p)

    rows = [dict( zip( heads, r)) for r in rows ]

    return rows

